

local Widget = LIBTD('Widget')
if not Widget or Widget('Editbox') then return end

local tonumber = tonumber
local CreateFrame = CreateFrame

local Editbox = Widget:NewModule('Editbox', CreateFrame('Editbox'))
Widget:CopyFunctions(Editbox)

function Editbox:New(parent, db, ...)
    local editbox = self:Bind(CreateFrame('Editbox', nil, parent))
    
    editbox:SetAutoFocus(nil)
    editbox:SetHeight(20)
    editbox:SetFontObject('ChatFontNormal')
    editbox:SetTextInsets(8, 8, 0, 0)
    editbox:Backdrop()
    
    editbox:SetScript('OnEscapePressed', self.Update)
    editbox:SetScript('OnEnterPressed', self.OnEnterPressed)
    editbox:SetScript('OnTextChanged', self.OnTextChanged)
    
    editbox.tdLabel = editbox:CreateLabel('BOTTOMLEFT', 'TOPLEFT', 0, 0)
    
    editbox:SetDBAndKeys(db, ...)
    
    return editbox
end

function Editbox:IsNumeric()
    return self.numeric
end

function Editbox:SetNumeric(state)
    self.numeric = state
end

function Editbox:SetValue(value)
    if self:IsNumeric() then
        self:SetNumber(tonumber(value) or 0)
    else
        self:SetText(value or '')
    end
end

function Editbox:GetValue()
    return self:IsNumeric() and self:GetNumber() or self:GetText()
end

function Editbox:GetPositionArgs()
    return 40, -15, 0, 0
end

function Editbox:Update()
    self:SetValue(self:GetDBValue())
    self:ClearFocus()
end

function Editbox:OnEnterPressed()
    local value = self:GetValue()
    if value then
        self:SaveDBValue(value)
    end
    self:ClearFocus()
end

local validnumber = {
    [''] = true,
    ['-'] = true,
    ['0.'] = true,
}

function Editbox:OnTextChanged()
    if not self:HasFocus() then
        self:SetCursorPosition(0)
        self:HighlightText(0, 100)
    end
    
    local text = self:GetText()
    if self:IsNumeric() and not validnumber[text] then
        local value = tonumber(text)
        if not value then
            self:SetText(self.prevValue)
            return
        elseif value == 0 then
            self.prevValue = 0
            self:SetText(0)
            return
        end
    end
    self.prevValue = text
    self:SetText(text)
end

function Editbox:SetLabelText(text)
    self.tdLabel:SetText(text)
end

function Editbox:GetWidgetType()
    return 'Editbox'
end

local Panel = Widget('Panel')
local ScrollEditbox = Widget:NewModule('ScrollEditbox', Panel:New())
Widget:CopyFunctions(ScrollEditbox, 'Backdrop')

local function EditboxOnCursorChanged(self, x, y, w, h)
	self.cursorOffset = y
	self.cursorHeight = h
	self.handleCursorChange = true
end

local function EditboxOnUpdate(self)
    local height, range, scroll, size, cursorOffset
	if self.handleCursorChange then
		local scrollFrame = self:GetParent()
        
		height = scrollFrame:GetHeight()
		range = scrollFrame:GetVerticalScrollRange()
		scroll = scrollFrame:GetVerticalScroll()
		size = height + range
		cursorOffset = -self.cursorOffset
		
		if ( math.floor(height) <= 0 or math.floor(range) <= 0 ) then
			return
		end
		
		while ( cursorOffset < scroll ) do
			scroll = (scroll - (height / 2))
			if ( scroll < 0 ) then
				scroll = 0
			end
			scrollFrame:SetVerticalScroll(scroll)
		end

		while ( (cursorOffset + self.cursorHeight) > (scroll + height) and scroll < range ) do
			scroll = (scroll + (height / 2))
			if ( scroll > range ) then
				scroll = range
			end
			scrollFrame:SetVerticalScroll(scroll)
		end
		self.handleCursorChange = false
	end
end

local function EditboxOnTextChanged(self)
    if self.readonly then
        self:SetText(self.text)
    else
        self.handleCursorChange = true
        EditboxOnUpdate(self, 0)
    end
end

function ScrollEditbox:New(parent)
    local panel = self:Bind(Panel:New(parent))
    panel:Backdrop()
    
    local scroll = Panel:New(panel)
    scroll:SetPoint('TOPLEFT', 5, -5)
    scroll:SetPoint('BOTTOMRIGHT', -5, 5)
    scroll:SetBackdrop{}
    scroll:SetScript('OnVerticalScroll', function(self, offset)
        self.scrollbar:SetValue(offset)
    end)
    
    local editbox = CreateFrame('Editbox', nil, scroll)
    editbox:SetFontObject('ChatFontNormal')
    editbox:SetTextInsets(4, 4, 4, 4)
    editbox:SetMultiLine(true)
    editbox:SetAutoFocus(false)
    editbox:SetPoint('TOPLEFT')
    editbox:SetScript('OnEscapePressed', editbox.ClearFocus)
    editbox:SetScript('OnTextChanged', EditboxOnTextChanged)
    editbox:SetScript('OnCursorChanged', EditboxOnCursorChanged)
    editbox:SetScript('OnUpdate', EditboxOnUpdate)
    
    scroll:EnableScroll(editbox, 1)
    panel:SetScript('OnSizeChanged', self.OnSizeChanged)
    
    panel.scroll = scroll
    panel.editbox = editbox
    
    return panel
end

function ScrollEditbox:OnSizeChanged(w, h)
    self.editbox:SetSize(w - 35, h - 10)
    self.scroll:SetWheelStep((h-10)/2)
    self.scroll:OnScrollRangeChanged()
end

function ScrollEditbox:SetText(text)
    self.editbox.text = text
    self.editbox:SetText(text)
end

function ScrollEditbox:GetText()
    return self.editbox:GetText()
end

function ScrollEditbox:SetFocus()
    self.editbox:SetFocus()
end

function ScrollEditbox:ClearFocus()
    return self.editbox:ClearFocus()
end

function ScrollEditbox:SetReadOnly(ro)
    self.editbox.readonly = ro
end
