
local LIB, VERSION = 'LIBTD', 8
local lib = _G[LIB]

if lib and lib.version >= VERSION then return end

local _G, type, print, format, pairs, ipairs, select, setmetatable = _G, type, print, format, pairs, ipairs, select, setmetatable
local GetLocale, GetAddOnMetadata = GetLocale, GetAddOnMetadata

if not lib then
    lib = {}
    _G[LIB] = lib
end

lib.version = VERSION
lib.addons = lib.addons or {}

local f = CreateFrame('Frame')
f:RegisterEvent('ADDON_LOADED')
f:SetScript('OnEvent', function(self, event, addon)
    local Addon = lib.addons[addon]
    if Addon and Addon.OnInit then
        Addon:OnInit()
    end
end)

function lib:Bind(t)
    return setmetatable(t, self.mt)
end

function lib:CopyTable(t)
    if not t or type(t) ~= 'table' then return t end
    
    local s = {}
    for k, v in pairs(t) do
        s[k] = self:CopyTable(v)
    end
    return s
end

function lib:GetName()
    return self.addonName
end

function lib:GetVersion()
    return self.version
end

function lib:RegisterCmd(func, ...)
    local name = strupper(self:GetName())
    for i = 1, select('#', ...) do
        _G['SLASH_' .. name .. i] = select(i, ...)
    end
    
    SlashCmdList[name] = func
end

------------------------------------------------------------------------ addons

local function getVersion(version)
    version = version and tostring(version)
    
    if not version then
        return 0, 0
    end
    
    local wow, ui = version:match('^(%d+).(%d+)$')
    return wow and tonumber(wow) or 0, ui and tonumber(ui) or 0
end

function lib:CheckVersion(oversion, version)
    if not version or not oversion then
        return true
    end
    
    if type(oversion) == 'number' then
        return tonumber(version) - oversion > 0
    else
        local owow, oui = getVersion(oversion)
        local nwow, nui = getVersion(version)
        
		return owow < nwow or (owow == nwow and oui < nui)
    end
end

function lib:NewAddon(name, table, version)
    if not name then return end
    
    table = table or {}
    
    self:Bind(table)
    
    table.locale = setmetatable({}, {__index = function(self, key) return key end})
    table.version = version or GetAddOnMetadata(name, 'Version')
    table.addonName = name
    table.modules = {}
    
    self.addons[name] = table
    
    return table
end

function lib:GetAddon(name)
    return self.addons[name]
end
setmetatable(lib, {__call = lib.GetAddon})

function lib:Print(...)
    print(format('|cff00ff00%s|r:', self.addonName), ...)
end

------------------------------------------------------------------------ modules

local function OnEvent(self, event, ...)
    if self[event] then
        self[event](self, ...)
    end
end

local function StartUpdate(self)
    if self.OnUpdate then
        self:SetScript('OnUpdate', self.OnUpdate)
        self:Show()
    end
end

local function StopUpdate(self)
    self:SetScript('OnUpdate', nil)
    self:Hide()
end

local MODULE_ARGS = {
    Event = function(self)
        self.OnEvent = OnEvent
        self:SetScript('OnEvent', OnEvent)
    end,
    
    Update = function(self)
        self.StartUpdate = StartUpdate
        self.StopUpdate = StopUpdate
    end,
}

function lib:NewModule(name, table, ...)
    if not name and type(name) ~= 'string' then return end
    
    if self[name] then
        return self[name]
    end
    
    table = table or {}
    table.mt = {__index = table}
    table.version = version
    
    local temp
    for i = 1, select('#', ...) do
		temp = select(i, ...)
        if MODULE_ARGS[temp] then
            MODULE_ARGS[temp](table)
        end
    end
    
    table.Bind = self.Bind
    
    self[name] = table
    self.modules[name] = table
    
    if table.Hide and type(table.Hide) == 'function' then table:Hide() end
    
    return table
end

function lib:GetModule(module)
    if not module or type(module) ~= 'string' then return end
    
    return self[module]
end

function lib:InitModule(module)
    module = self:GetModule(module)
    
    if module and module.Init then
        module:Init()
    end
end

------------------------------------------------------------------------ locales

function lib:NewLocale(locale)
    if GetLocale() ~= locale then
        return
    end
    return self.locale
end

function lib:GetLocale()
    return self.locale
end

lib.mt = {__index = lib, __call = lib.GetModule}
